from typing import Type, TypeVar, Literal, Optional
import httpx
from mcp.client.streamable_http import RequestContext
from pydantic import BaseModel
from mcp.server.fastmcp import FastMCP, Context

from .tools import register_all_tools
from .u_tool import register_u_tool


def serve(
    api_base: str,
    transport: Literal["stdio", "sse", "streamable-http"],
    twelve_data_apikey: Optional[str],
    number_of_tools: int,
    u_tool_open_ai_api_key: Optional[str],
    u_tool_oauth2: bool
) -> None:
    # logger = logging.getLogger(__name__)

    server = FastMCP(
        "mcp-twelve-data",
        host="0.0.0.0",
        port="8000",
    )

    P = TypeVar('P', bound=BaseModel)
    R = TypeVar('R', bound=BaseModel)

    async def _call_endpoint(
        endpoint: str,
        params: P,
        response_model: Type[R],
        ctx: Context
    ) -> R:
        if transport == 'stdio' and twelve_data_apikey:
            params.apikey = twelve_data_apikey
        elif transport == "streamable-http" and twelve_data_apikey:
            params.apikey = twelve_data_apikey
        else:
            rc: RequestContext = ctx.request_context
            apikey_header = rc.headers.get('authorization')
            split_header = apikey_header.split(' ') if apikey_header else []
            if len(split_header) == 2:
                params.apikey = split_header[1]

        async with httpx.AsyncClient(
            trust_env=False,
            headers={
                "accept": "application/json",
                "user-agent": "python-httpx/0.24.0"
            },
        ) as client:
            resp = await client.get(
                f"{api_base}/{endpoint}",
                params=params.model_dump(exclude_none=True)
            )
            resp.raise_for_status()
            return response_model.model_validate(resp.json())

    register_all_tools(server=server, _call_endpoint=_call_endpoint)

    if u_tool_oauth2 or u_tool_open_ai_api_key is not None:
        # if u_tool_oauth2 is True, then u_tool_open_ai_api_key is None and will be received via Twelve data server
        register_u_tool(
            server=server,
            u_tool_open_ai_api_key=u_tool_open_ai_api_key,
            transport=transport
        )
    else:
        all_tools = server._tool_manager._tools
        server._tool_manager._tools = dict(list(all_tools.items())[:number_of_tools])

    server.run(transport=transport)



from typing import Literal, Optional

import click
import logging
import sys
from .server import serve


@click.command()
@click.option("-v", "--verbose", count=True)
@click.option("-t", "--transport", default="stdio", help="stdio, streamable-http")
@click.option(
    "-k",
    "--twelve-data-apikey",
    default=None,
    help=(
        "This parameter is required for 'stdio' transport. "
        "For 'streamable-http', you have three options: "
        "1. Use the -k option to set a predefined API key. "
        "2. Use the -ua option to retrieve the API key from the Twelve Data server. "
        "3. Provide the API key in the 'Authorization' header as: 'apikey <your-apikey>'."
    )
)
@click.option(
    "-n", "--number-of-tools", default=35,
    help="limit number of tools to prevent problems with mcp clients, max n value is 193, default is 35"
)
@click.option(
    "-u", "--u-tool-open-ai-api-key", default=None,
    help=(
        "If set, activates a unified 'u-tool' powered by OpenAI "
        "to select and call the appropriate Twelve Data endpoint."
    ),
)
@click.option(
    "-ua", "--u-tool-oauth2", default=False, is_flag=True,
    help=(
        "If set, activates the unified 'u-tool' powered by OpenAI, "
        "and fetches Twelve Data and OpenAI API keys directly from the Twelve Data server."
    )
)
def main(
    verbose: bool,
    transport: Literal["stdio", "sse", "streamable-http"] = "stdio",
    twelve_data_apikey: Optional[str] = None,
    number_of_tools: int = 30,
    u_tool_open_ai_api_key: Optional[str] = None,
    u_tool_oauth2: bool = False,
) -> None:
    logging_level = logging.WARN
    if verbose == 1:
        logging_level = logging.INFO
    elif verbose >= 2:
        logging_level = logging.DEBUG

    logging.basicConfig(level=logging_level, stream=sys.stderr)

    if u_tool_oauth2 and u_tool_open_ai_api_key is not None:
        RuntimeError("Set either u_tool_open_ai_api_key or u_tool_oauth2")
    if u_tool_oauth2 and transport != "streamable-http":
        RuntimeError("Set transport to streamable-http if you want to use -ua option")
    if transport == "stdio" and twelve_data_apikey is None:
        RuntimeError("Set -k to use stdio transport")

    serve(
        api_base="https://api.twelvedata.com",
        transport=transport,
        twelve_data_apikey=twelve_data_apikey,
        number_of_tools=number_of_tools,
        u_tool_open_ai_api_key=u_tool_open_ai_api_key,
        u_tool_oauth2=u_tool_oauth2,
    )


if __name__ == "__main__":
    main()
